
#include "project.h"

uint8_t Length;
uint8_t CheckSum;
uint8_t Counter;
uint16_t DataSum;


//IPod Simple Remote Protocol:
//Referece: http://www.adriangame.co.uk/ipod-acc-pro.html
//Message Format:
// 0xFF, 0x55, Length, Mode, Command, Parameter(s), Checksum
// <-Header->
//                     <---------Length------->
//             <----------Checksum------------>
//CheckSum: 0x100 - ((Length + Mode + Command + Parameter(s)) & 0xFF)

const uint8_t IPodSwitchToMode2[] = {0x03, 0x00, 0x01, 0x02};
const uint8_t IPodToggleShuffle[] = {0x04, 0x02, 0x00, 0x00, 0x80};
const uint8_t IPodToggleRepeat[] = {0x05, 0x02, 0x00, 0x00, 0x00, 0x01};
const uint8_t IPodScrollUp[] = {0x06, 0x02, 0x00, 0x00, 0x00, 0x00, 0x01};
const uint8_t IPodScrollDown[] = {0x06, 0x02, 0x00, 0x00, 0x00, 0x00, 0x02};
const uint8_t IPodScipLeft[] = {0x03, 0x02, 0x00, 0x10};
const uint8_t IPodScipRight[] = {0x03, 0x02, 0x00, 0x08};
const uint8_t IPodOKSelect[] = {0x05, 0x02, 0x00, 0x00, 0x00, 0x80};
const uint8_t IPodMenu[] = {0x05, 0x02, 0x00, 0x00, 0x00, 0x40};
const uint8_t IPodPlay[] = {0x04, 0x02, 0x00, 0x00, 0x01};
const uint8_t IPodPause[] = {0x04, 0x02, 0x00, 0x00, 0x02};
const uint8_t IPodBtnRelease[] = {0x03, 0x02, 0x00, 0x00};

void SendShortPressCommand(uint8_t Address, uint8_t Command)
{
	if(Address == BASE_ADDRESS)
	{
		switch(Command)
		{
		case BTN_RPT:
		{
			BuildIPodMessage((uint8_t*)IPodToggleRepeat);
			break;
		}
		case BTN_LEFT:
		{
			BuildIPodMessage((uint8_t*)IPodScipLeft);
			break;
		}
		case BTN_RIGHT:
		{
			BuildIPodMessage((uint8_t*)IPodScipRight);
			break;
		}
		case BTN_UP:
		{
			BuildIPodMessage((uint8_t*)IPodScrollUp);
			break;
		}
		case BTN_DOWN:
		{
			BuildIPodMessage((uint8_t*)IPodScrollDown);
			break;
		}
		case BTN_MAP_MENU:
		{
			BuildIPodMessage((uint8_t*)IPodMenu);
			break;
		}
		case BTN_MINUS:
		{
			BuildIPodMessage((uint8_t*)IPodPause);
			break;
		}
		case BTN_PLUS:
		{
			BuildIPodMessage((uint8_t*)IPodPlay);
			break;
		}
		case BTN_OK:
		{
			BuildIPodMessage((uint8_t*)IPodOKSelect);
			break;
		}
		default: break;
		}
		BuildIPodMessage((uint8_t*)IPodBtnRelease);
	}

}

void SendLongPressCommand(uint8_t Address, uint8_t Command)
{
	if(Address == BASE_ADDRESS)
	{
		switch(Command)
		{
		case BTN_RPT:
		{
			BuildIPodMessage((uint8_t*)IPodToggleShuffle);
			BuildIPodMessage((uint8_t*)IPodBtnRelease);
			break;
		}
		case BTN_MAP_MENU:
		{
			P1OUT ^= 0x01;
		}
		default: break;
		}
	}

}

void BuildIPodMessage(uint8_t* Message)
{
	Length = *Message;		//Lnge der Daten = 1. Feld in Message
	Length += 1;			//+1 fr das Length-Byte

	PutSingleChar(0xFF);
	PutSingleChar(0x55);

	DataSum = 0;
	for(Counter = 0; Counter < Length; Counter++)		//Datenfeld senden
	{
		DataSum += *Message;							//fr Checksumme aufsummieren
		PutSingleChar(*Message);
		Message++;
	}

	DataSum &= 0xFF;									//LowByte der Checksumme
	CheckSum = (uint8_t)(0x100 - DataSum);				//Checksumme berechnen und senden
	PutSingleChar(CheckSum);
}
