//***************************************************************************************************
// RC5 Empfnger zur Steuerung des Apple IPod
// Autor: Christian Richter
// Basierend auf den TI-Application Notes fr RC5 Empfang und Software-UART
// Verwendeter Prozessor: MSP430F1121, es kann jedoch jeder MSP430 mit Timer A verwendet werden.
// Es werden alle 3 CCR des Timer A verwendet.
// Frei fr private Nutzung
//***************************************************************************************************

#include "project.h"

volatile uint8_t i;
extern volatile unsigned int SYSTEM_FLAGS;
extern volatile unsigned char RX_Bytes[16];
extern volatile unsigned char charcount;
extern volatile uint16_t IRData;

volatile uint8_t Command;
volatile uint8_t Address;
volatile uint8_t Toggle;

volatile uint8_t LastToggle;

volatile uint16_t CommandCounter = 0;
volatile uint8_t NewIRCommand = 0;

int main(void)
{

	//Watchdog deaktivieren
	WDTCTL = WDTHOLD + WDTPW;

	//XT1 aktivieren und auf Stabilitt testen
	BCSCTL1 |= XTS + DIVA_2;                           // ACLK = LFXT1 = (HF XTAL / 4) = 921.600 kHz
	do
	{
	IFG1 &= ~OFIFG;                           // Clear OSCFault flag
	for (i = 0xFF; i > 0; i--);               // Time for flag to set
	}
	while ((IFG1 & OFIFG));                   // OSCFault flag still set?

	BCSCTL2 |= SELM_3;                        // MCLK = LFXT1 (safe)

	CCTL0 = OUT;                              // TXD Idle as Mark
	TACTL = TASSEL_1 + MC_2 + TACLR + ID_1;   // ACLK, continuous mode, Teiler :2 -> TACLK = 460,8kHz

	P1DIR = 0x03;		//P1.0 AUsgang LED, P1.1 Ausgang Serial Data
	P1OUT &= ~0x01;		//LED ausschalten
	P1SEL = 0x02;		//P1.1 ist Timer A0 Ausgang Software UART TX
	P2SEL = 0x04;		//P2.2 ist Timer A0 Eingang Software UART RX
	P2DIR = 0x01;		//P2.0 ist Ausgang fr Autoradio-Umschaltung
	P2OUT &= ~0x01;		//Autoradio-Ausgang auf 0

	WDTCTL = WDT_MDLY_32;	//~32ms Timer-Intervall WDT, Abtastung ext. Schaltsignale
	IE1 |= WDTIE;                             // Enable WDT interrupt

	_EINT();			//Globalen Interrupt freigeben, sonst weder Senden noch Empfangen per Software UART

	IR_Ready();			//IR-Empfang ein

	while(1)
	{
		if(SYSTEM_FLAGS)	//Was zu tun?
		{

			if(SYSTEM_FLAGS & IR_RX_FLAG)			//Ein kompletter RC5-Code wurde empfangen
			{

				//P1OUT |= 0x01;
				Command = (uint8_t)IRData & 0x3F;			//Dekodieren
				Address = (uint8_t)((IRData >> 6) & 0x1F);
				Toggle = (uint8_t)((IRData >> 11) & 0x01);

				LastToggle = Toggle;

				NewIRCommand = 1;

				//Kommando-Zhler fr Erkennung kurzer/langer Tastendruck
				if(Toggle == LastToggle)CommandCounter++;
				else CommandCounter = 0;

				SYSTEM_FLAGS &= ~IR_RX_FLAG;
				//Etwas warten, weil der PC die Kommandos sonst verschluckt
				Wait(10000);
				IR_Ready();		//Bereit fr neues IR-Kommando
				//P1OUT &= ~0x01;
				_EINT();		// Int wieder ein

			}
			if(SYSTEM_FLAGS & TIMER_FLAG)					//Timer-Abtastung
			{
				if((!NewIRCommand) && (CommandCounter))		//Ende des Tastendruckes
				{

					if(CommandCounter > 3)					//langer Tastendruck erkannt
					{
						//SendLongPressCommand(Address, Command);
					}

					else									//kurzer Tastendruck erkannt
					{
						SendShortPressCommand(Address, Command);
					}
					CommandCounter = 0;						//Bereit fr neuen RC5-Befehl
				}

				if(CommandCounter == 4)
				{
					SendLongPressCommand(Address, Command);
				}

				SYSTEM_FLAGS &= ~TIMER_FLAG;

				NewIRCommand = 0;

			}
		}
		_EINT();
	}

	return 0;
}
